from wrapmail.smtp.Smtp_Sender import SMTP_Sender
from wrapmail.abstract_mail import Mail


class SMTP_mail(Mail):
    def __init__(self, TO, TITLE="", MSG="", html=None):
        sender = SMTP_Sender()
        super().__init__(sender, TO, TITLE, MSG, html)




from wrapmail.abstract_sender import Abstract_Sender
from wrapmail.gmail.Google import Create_Service
import base64

class Gmail_Sender(Abstract_Sender):
    def __init__(self):
        super().__init__()

    def start_service(self):
        try:
            self.service = Create_Service(
                "client_secret.json",
                "gmail",
                "v1",
                ["https://mail.google.com/"]
            )
            self.available = True
        except Exception as e:
            print("could not start gmail service")
            print(e)
            self.available = False

    def send(self, mail):
        super().send(mail)
        if self.available == False:
            print("service unavailable")
            return False

        raw_string = base64.urlsafe_b64encode(
            mail.as_bytes()).decode()
        self.service.users().messages().send(
            userId='me', body={'raw': raw_string}).execute()
        print("email successfully sent")
        return True



from abc import ABC, abstractmethod
from wrapmail.abstract_sender import Abstract_Sender
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
import mimetypes
from email import encoders
from email.mime.base import MIMEBase
import os


class Mail(ABC):
    def __init__(self, sender, TO, TITLE="", MSG="", html=None):

        self.html = html
        self.TO = TO
        self.MSG = MSG
        self.TITLE = TITLE
        self.mail = MIMEMultipart()
        self.sender = sender

        self.mail["To"] = self.TO
        self.mail["Subject"] = self.TITLE

        if (html is None):
            body = MIMEText(self.MSG, "plain")
            self.mail.attach(body)
        else:
            with open(self.html, "r") as file:
                message = file.read()
                body = MIMEText(message, 'html')
                self.mail.attach(body)


    def send(self) -> bool:
        self.sender.send(self.mail)


    def add_attachment(self, attach) -> bool:

        content_type, encoding = mimetypes.guess_type(attach)
        main_type, sub_type = content_type.split('/',1)
        file_name = os.path.basename(attach)
        with open(attach, 'rb') as f:
            myFile = MIMEBase(main_type, sub_type)
            myFile.set_payload(f.read())
            myFile.add_header('Content-Disposition','attachment',filename=file_name)
            encoders.encode_base64(myFile)
        self.mail.attach(myFile)
        return True



from abc import ABC, abstractmethod

class Abstract_Sender(ABC):
    @abstractmethod
    def __init__(self):
        pass

    @abstractmethod
    def start_service(self):
        pass
    
    @abstractmethod
    def send(self, mail):
        self.start_service()


from wrapmail.abstract_sender import Abstract_Sender
import os
import smtplib
import traceback

class SMTP_Sender(Abstract_Sender):
    def __init__(self):
        super().__init__()

    def start_service(self):
        try:
            self.connection = smtplib.SMTP("smtp.gmail.com", 587)
            self.connection.ehlo()
            self.connection.starttls()
            self.connection.login(os.environ.get("EMAIL"),
                              os.environ.get("PASSWORD"))
            self.available = True
        except Exception as e:
            print("could not start smtp service")
            print(e) 
            self.available = False

    def send(self, mail):
        super().send(mail)
        if self.available == False:
            print("service unavailable")
            return False

        try:
            self.start_service()
            self.connection.sendmail(
                os.environ.get("EMAIL"),
                mail["To"],
                mail.as_string()
            )

            self.connection.quit()

        except Exception:
            print("could not send the mail")
            traceback.print_exc()
            return False


from setuptools import setup, find_packages
import codecs
import os
import pathlib
import pkg_resources

here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()

VERSION = '0.0.1'
DESCRIPTION = 'A package that helps to send emails through Python utilizing an OOP logic'
LONG_DESCRIPTION = 'A package that helps to send emails through Python utilizing an OOP logic. Includes a wrapper for Gmail API and an SMTP-based system'

with pathlib.Path('requirements.txt').open() as requirements_txt:
    install_requires = [
        str(requirement)
        for requirement
        in pkg_resources.parse_requirements(requirements_txt)
    ]

# Setting up
setup(
    name="wrapmail",
    version=VERSION,
    author="Alperen Yıldız",
    author_email="<alperenyildiz@sabanciuniv.edu",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=long_description,
    packages=find_packages(),
    install_requires=install_requires,
    keywords=['python', 'mail','gmail','api','smtp'],
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)